#include <stdio.h>
#include <stdlib.h>
#include <sndfile.h>
#include <alsa/asoundlib.h>
#include <pthread.h>
#include "snd_header.h"
#define MAXTHREAD 30

static char *device = "default"; /* playback device */
int kb=0;
pthread_mutex_t mutex;
int loop=1;

void conf_pcm(snd_pcm_t *handle, int channels, int samplerate){
	int err;
	if ((err = snd_pcm_set_params(handle, SND_PCM_FORMAT_S16, SND_PCM_ACCESS_RW_INTERLEAVED, channels, samplerate*2, 1, 500000)) < 0) { /* 0.5sec */
		printf("Playback open error: %s\n", snd_strerror(err));
		exit(EXIT_FAILURE);
	}

}

SF_INFO readSndFile(char *arg, int **ptr, int *frames){
	SNDFILE *snd_file;
	SF_INFO sfinfo;
	snd_file=sf_open(arg, SFM_READ, &sfinfo);
	if(sf_error(snd_file)!=0){
		perror("sf_open");
		exit(EXIT_FAILURE);	
	}

	*ptr=(int *) malloc(sfinfo.frames*sfinfo.channels*sizeof(int));
	*frames=(int) sf_readf_int(snd_file, *ptr, sfinfo.frames);
	sf_close(snd_file);
#ifdef DEBUG
	printf("Frames read : %d Ch : %d\n", *frames, sfinfo.channels);	
#endif
	return sfinfo;
}

void fillSndData(sndData *snd){
	int i;
	char files[FILECOUNT][MAXCHAR] = {"wav_files/triangle.wav", "wav_files/CB.wav", "wav_files/cymbal.wav", "wav_files/drum.wav", "wav_files/sitar.wav", "wav_files/electric_guitar.wav"};
	for(i=0; i<FILECOUNT;i++){
		snd[i].sfinfo=readSndFile(files[i],&(snd[i].data), &(snd[i].frames));//read audio data
		snd[i].repeat=1;
	}
}


void writei_pcm(snd_pcm_t *handle, int *data, int frm, int repeat){
	int i,err=0;
	snd_pcm_sframes_t frames;
	for (i = 0; i < repeat; i++) {
		frames = snd_pcm_writei(handle,data, 2*frm);
		if (frames < 0){
			frames = snd_pcm_recover(handle, frames, 0);
			printf("recover\n");
		}
		if (frames < 0) {
			printf("snd_pcm_writei failed: %s\n", snd_strerror(err));
			break;
		}
		if (frames > 0 && frames <frm*2)
			printf("Short write (expected %li, wrote %li)\n", (long)frm*2, frames);
	}

}

void changeVolume(long volume){
	long min, max;
	snd_mixer_t *handle;
	snd_mixer_selem_id_t *sid;
	//const char *card = "default";
	const char *selem_name = "softvol";

	snd_mixer_open(&handle, 0);
	snd_mixer_attach(handle,device);
	snd_mixer_selem_register(handle, NULL, NULL);
	snd_mixer_load(handle);

	snd_mixer_selem_id_alloca(&sid);
	snd_mixer_selem_id_set_index(sid, 0);
	snd_mixer_selem_id_set_name(sid, selem_name);
	snd_mixer_elem_t* elem = snd_mixer_find_selem(handle, sid);

	snd_mixer_selem_get_playback_volume_range(elem, &min, &max);
#ifdef DEBUG
	printf("Min volume range : %lu\nMax volume range: %lu\n", min, max);
#endif
	snd_mixer_selem_set_playback_volume_all(elem, volume * max / 100);

	snd_mixer_close(handle);
}

void *sndThread(void *arg){
	while(nbThread>MAXTHREAD-2){
		usleep(50000);
	}
	pthread_mutex_lock(&mutex);
	nbThread++;
	pthread_mutex_unlock(&mutex);
	int status, err;
	snd_pcm_t *handle;
	sndData snd=*((sndData *) arg);	
	
	
	if ((err = snd_pcm_open(&handle, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
			printf("Playback open error: %s\n", snd_strerror(err));
			exit(EXIT_FAILURE);
	}
	
	conf_pcm(handle, snd.sfinfo.channels, snd.sfinfo.samplerate);
	//changeVolume(20);
	writei_pcm(handle, snd.data, snd.frames, snd.repeat);
	
	snd_pcm_close(handle);
	pthread_mutex_lock(&mutex);
	nbThread--;
	pthread_mutex_unlock(&mutex);
	pthread_exit((void *)&status);
}


void soundLoop(sndData *snd, int val){	
	srand(time(NULL));
	pthread_mutex_init(&mutex, NULL);
	int err;
	pthread_t tid;
	pthread_attr_t attr;
	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	
	if(nbThread<MAXTHREAD && val>=0 && val<NB ){								
		err = pthread_create(&tid, &attr, sndThread, (void *)&snd[val]);
		if(err){perror("Error pthread");exit(EXIT_FAILURE);}
	}
}


void freeData(sndData *snd){
	int i;
	for(i=0; i<FILECOUNT;i++){
		free(snd[i].data);
	}

}
